#!/usr/bin/php -q
<?php
# This file is part of Website@School, a Content Management System especially designed for schools.
# Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker <peter@berestijn.nl>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3 as published by
# the Free Software Foundation supplemented with the Additional Terms, as set
# forth in the License Agreement for Website@School (see /program/license.html).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License
# for more details.
#
# You should have received a copy of the License Agreement for Website@School
# along with this program. If not, see http://websiteatschool.eu/license.html

/** mimemerge.php - merge new and existing mappings between extension and mimetypes
 *
 * This file contains a quick and dirty tool to merge existing mappings
 * (from file extension to mimetype and vice versa) with a newer revision of
 * the Apache mime.types file. Note that this file placed in the public domain
 * for unlimited distribution, see the headers in  {@link mime.types}.
 *
 * The merged mappings are to be included in the file {@filelib.php}.
 * This program generates a code snippet that can easily be imported into
 * {@link filelib.php}, replacing the existing table(s).
 *
 * Usage is: mimemerge.php file [file ...] > snippet.php
 *
 * The resulting file snippet.php is supposed to be incorporated into
 * filelib.php (manually).
 *
 * The code in snippet.php boils down to the definition of two arrays:
 * one to map a file extension to one or more mime types, and
 * one to map a mime type to one or more file extensions.
 * If a mapping is 1-to-1 the array element is a string, if the mapping
 * is 1-to-N the element is an array of strings. Note that all string
 * values are US-ASCII, hence they can be converted to lowercase without
 * worries about UTF-8.
 *
 * The resulting arrays are sorted by key.
 *
 * Note:
 * The empty extension '' is a special case: it maps to a generic 
 * 'application/octet-stream'. In the reverse case it is the first extension
 * in the entry for the mime type.
 * Also there is a 'generic' extension 'bin' which is the second extension
 * in the list for 'application/octet-stream'. Note that these two extensions
 * should be the first two entries.
 *
 * The public domain file mime.types is maintained  by the Apache Foundation
 * and the latest version of the file can be downloaded from
 * https://svn.apache.org/viewvc/httpd/httpd/trunk/docs/conf/mime.types?view=co
 * The current version is also added to the Website@School repository (see
 * ../imports/mime.types), for reference.
 *
 * @author Peter Fokker <peter@berestijn.nl>
 * @copyright Copyright (C) 2008-2016 Ingenieursbureau PSD/Peter Fokker
 * @license http://websiteatschool.eu/license.html GNU AGPLv3+Additional Terms
 * @package wascore
 * @version $Id: mimemerge.php,v 1.1 2016/06/02 16:24:52 peter Exp $
 */
define('WASENTRY',__FILE__);

$filelib = dirname(WASENTRY).'/../../websiteatschool/program/lib/filelib.php';
require_once($filelib);

$source = basename($filelib);
$prog = basename($argv[0]);

// 1 - get original array(s) in core
$mime_cur = array();
$mime_ext = array('' => array('application/octet-stream'),'bin' => array('application/octet-stream'));
$mime_typ = array('application/octet-stream' => array('','bin'));
global $MIME_EXT_TYP;
global $MIME_TYP_EXT;

if ((!empty($MIME_EXT_TYP)) && (!empty($MIME_EXT_TYP))) {
    // filelib already converted to the new regime; use the two global tables as a starting point
    foreach($MIME_EXT_TYP as $ext => $v) {
        if (!is_array($v)) {
            $v = array($v);
        }
        foreach($v as $typ) {
            if (!isset($mime_ext[$ext])) {
                $mime_ext[$ext] = array($typ);
            } elseif (!in_array($typ,$mime_ext[$ext])) {
                $mime_ext[$ext][] = $typ;
            }
        }
    }
    foreach($MIME_TYP_EXT as $typ => $v) {
        if (!is_array($v)) {
            $v = array($v);
        }
        foreach($v as $ext) {
            if (!isset($mime_typ[$typ])) {
                $mime_typ[$typ] = array($ext);
            } elseif (!in_array($ext,$mime_typ[$typ])) {
                $mime_typ[$typ][] = $ext;
            }
        }
    }
    $source .= ':arrays()';
} elseif (function_exists('get_mimetypes_array')) {
    // filelib still working with the old reqime; use the existing function to populate the two arrays
    $source .= ':function()';
    $mime_cur = get_mimetypes_array();
    foreach($mime_cur as $ext => $typ) {
        $ext = strtolower($ext);
        $typ = strtolower($typ);
        if (!isset($mime_ext[$ext])) {
            $mime_ext[$ext] = array($typ);
        } elseif (!in_array($typ,$mime_ext[$ext])) {
            $mime_ext[$ext][] = $typ;
        }
        if (!isset($mime_typ[$typ])) {
            $mime_typ[$typ] = array($ext);
        } elseif (!in_array($ext,$mime_typ[$typ])) {
            $mime_typ[$typ][] = $ext;
        }
    }
} else {
    fwrite(STDERR,sprintf("%s: cannot find original mimetype mapping(s); exiting...\n",$prog));
    exit(1);
}

// 2 -- just a reminder of the status quo
fwrite(STDERR,sprintf("ext=>typ: %d, type=>ext: %d [%s]\n",sizeof($mime_ext),sizeof($mime_typ),$source));

// 3 -- add from additional mime.types file(s)
for ($i=1; $i < $argc; ++$i) {
    $file = $argv[$i];
    if (($fp = fopen($file,"r")) === FALSE) {
        fwrite(STDERR,sprintf("%s: error: cannot open %s; skipping\n",$prog,$file));
        continue;
    }
    $exts = $typs = 0;
    while (!feof($fp)) {
        $buffer = trim(fgets($fp,4096));
        if ((empty($buffer)) || (preg_match('/^\s*#/',$buffer))) { // skip emptylines, comment lines
            continue;
        }
        $tokens = preg_split('/\\s+/',$buffer);

        $typ = strtolower($tokens[0]);
        for ($j=1; $j<sizeof($tokens); ++$j) {
            $ext = strtolower($tokens[$j]);

            if (!isset($mime_ext[$ext])) {
                $mime_ext[$ext] = array($typ);
                ++$exts;
            } elseif (!in_array($typ,$mime_ext[$ext])) {
                $mime_ext[$ext][] = $typ;
            }
            if (!isset($mime_typ[$typ])) {
                $mime_typ[$typ] = array($ext);
                ++$typs;
            } elseif (!in_array($ext,$mime_typ[$typ])) {
                $mime_typ[$typ][] = $ext;
            }
        }
    }
    fclose($fp);
    fwrite(STDERR,sprintf("ext=>typ: %d, type=>ext: %d [%s]\n",$exts,$typs,basename($file)));
}
fwrite(STDERR,sprintf("ext=>typ: %d, type=>ext: %d [%s]\n",sizeof($mime_ext),sizeof($mime_typ),'total'));

// 4 -- construct actual code snippet
$fp = STDOUT;
ksort($mime_ext);
ksort($mime_typ);

$table = array('MIME_EXT_TYP' => $mime_ext,'MIME_TYP_EXT' => $mime_typ);
foreach($table as $name => $lookup) {
    fwrite($fp,sprintf("global \$%s;\n".
                       "if (empty(\$%s)) { \$%s=array(",$name,$name,$name));
    $i = 0;
    foreach($lookup as $k => $v) {
        if (sizeof($v) == 1) { // plain string will do
            fwrite($fp,sprintf("%s\n '%s'=>'%s'",($i++) ? ',':'', $k,$v[0]));
        } else {
            fwrite($fp,sprintf("%s\n '%s'=>array('%s')",($i++) ? ',':'',$k,implode("','",$v)));
        }
    }
    fwrite($fp,sprintf("\n );\n}\n"));
}
fclose($fp);

?>